window.Gleam.initSearch([{"doc":"paint","title":"paint","content":"# Paint\n**Make drawings, animations, and games with Gleam!**\n\n[![Package Version](https://img.shields.io/hexpm/v/paint)](https://hex.pm/packages/paint)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/paint/)\n\nPaint is a tiny embedded domain specific language (inspired by [Gloss](https://hackage.haskell.org/package/gloss)).\nMake pictures out of basic shapes then style, transform, and combine them using the provided functions.\n\n![Frame 3(2)](https://github.com/user-attachments/assets/a8b83b58-990a-432a-9034-deebc4d210a6)\n\n\n```gleam\nimport paint as p\nimport paint/canvas\n\nfn main() {\n  let my_picture = p.combine([\n    p.circle(50.0),\n    p.circle(30.0) |&gt; p.fill(p.colour_rgb(0, 200, 200)),\n    p.rectangle(100.0, 50.0) |&gt; p.rotate(p.angle_deg(30.0)),\n    p.text(&quot;Hello world&quot;, 20) |&gt; p.translate_y(-65.0),\n  ])\n\n  canvas.display(fn(_: canvas.Config) { my_picture }, &quot;#canvas_id&quot;)\n}\n```\n\n**Want to learn more? Read the [docs](https://hexdocs.pm/paint) or browse the [visual examples](https://adelhult.github.io/paint/).**\n\n&gt; [!NOTE]\n&gt; Currently, there is only a HTML canvas backend for displaying the pictures.\n&gt; Hopefully, I will have time to add other backends in the future (such as SVG). Feel free to contribute!\n\n## Logo\nLucy is borrowed from the [Gleam branding page](https://gleam.run/branding/) and the brush is made by [Delapouite (game icons)](https://game-icons.net/1x1/delapouite/paint-brush.html).\n\n## Changelog\nThe API is considered unstable until a `1.*.*` release is published.\nChanges between versions can be found in the file `CHANGELOG.md`.\n\n## TODOs\n- [ ] Would be fun to investigate an API more similar to [Haskell diagrams](https://hackage.haskell.org/package/diagrams) and [Diagrammer](https://www.youtube.com/watch?v=gT9Xu-ctNqI).\n- [x] Replace the color functions with [gleam-community/colour](https://hexdocs.pm/gleam_community_colour/)\n- [ ] Support (bitmap) images\n- [x] Split the library into multiple well structured modules (`paint`, `canvas`, `event`, etc.)\n- [x] Allowing arbitrary css selectors instead of requiring ids in `interact_...` and `display_...`\n- [x] Improve input handling for `canvas.interact` (more keys and mouse input)\n- [ ] Add another backend? SVG or maybe Raylib bindings (for either erlang or node).\n- [ ] Write a proper tutorial (and a nicer examples page).\n- [ ] More shapes, ellipse for example\n- [ ] (Maybe) support gradient fill\n","url":"index.html"},{"doc":"paint","title":"Angle","content":"pub type Angle =\n  internal_implementation.Angle\n An angle in clock-wise direction.\n See: `angle_rad` and `angle_deg`.\n\nSynonyms:\npaint.Angle\npaint Angle","url":"paint.html#Angle"},{"doc":"paint","title":"Colour","content":"pub type Colour =\n  colour.Colour\n A rexport of the Colour type from [gleam_community/colour](https://hexdocs.pm/gleam_community_colour/).\n Paint also includes the functions `colour_hex` and `colour_rgb` to\n easily construct Colours, but feel free to import the `gleam_community/colour` module\n and use the many utility that are provided from there.\n\nSynonyms:\npaint.Colour\npaint Colour","url":"paint.html#Colour"},{"doc":"paint","title":"Picture","content":"pub type Picture =\n  internal_implementation.Picture\n A 2D picture. This is the type which this entire library revolves around.\n\n&gt; [!NOTE]\n&gt; Unless you intend to author a new backend you should **consider this type opaque and never use any of its constructors**.\n&gt; Instead, make use of the many utility functions defined in this module (`circle`, `combine`, `fill`, etc.)\n\nSynonyms:\npaint.Picture\npaint Picture","url":"paint.html#Picture"},{"doc":"paint","title":"Vec2","content":"pub type Vec2 =\n  #(Float, Float)\n\n\nSynonyms:\npaint.Vec2\npaint Vec2","url":"paint.html#Vec2"},{"doc":"paint","title":"angle_deg","content":"pub fn angle_deg(degrees: Float) -&gt; Angle\n Create an angle expressed in degrees\nSynonyms:\npaint.angle_deg\npaint angle_deg","url":"paint.html#angle_deg"},{"doc":"paint","title":"angle_rad","content":"pub fn angle_rad(radians: Float) -&gt; Angle\n Create an angle expressed in radians\nSynonyms:\npaint.angle_rad\npaint angle_rad","url":"paint.html#angle_rad"},{"doc":"paint","title":"arc","content":"pub fn arc(radius: Float, start: Angle, end: Angle) -&gt; Picture\n An arc with some radius going from some\n starting angle to some other angle in clock-wise direction\nSynonyms:\npaint.arc\npaint arc","url":"paint.html#arc"},{"doc":"paint","title":"blank","content":"pub fn blank() -&gt; Picture\n A blank image\nSynonyms:\npaint.blank\npaint blank","url":"paint.html#blank"},{"doc":"paint","title":"circle","content":"pub fn circle(radius: Float) -&gt; Picture\n A circle with some given radius\nSynonyms:\npaint.circle\npaint circle","url":"paint.html#circle"},{"doc":"paint","title":"colour_hex","content":"pub fn colour_hex(string: String) -&gt; Colour\n A utility around [colour.from_rgb_hex_string](https://hexdocs.pm/gleam_community_colour/gleam_community/colour.html#from_rgb_hex_string)\n (from `gleam_community/colour`) that **panics** on an invalid hex code.\nSynonyms:\npaint.colour_hex\npaint colour_hex","url":"paint.html#colour_hex"},{"doc":"paint","title":"colour_rgb","content":"pub fn colour_rgb(red: Int, green: Int, blue: Int) -&gt; Colour\n A utility around [colour.from_rgb255](https://hexdocs.pm/gleam_community_colour/gleam_community/colour.html#from_rgb255)\n (from `gleam_community/colour`) that **panics** if the values are outside of the allowed range.\nSynonyms:\npaint.colour_rgb\npaint colour_rgb","url":"paint.html#colour_rgb"},{"doc":"paint","title":"combine","content":"pub fn combine(pictures: List(Picture)) -&gt; Picture\n Combine multiple pictures into one\nSynonyms:\npaint.combine\npaint combine","url":"paint.html#combine"},{"doc":"paint","title":"concat","content":"pub fn concat(\n  picture: Picture,\n  another_picture: Picture,\n) -&gt; Picture\n Concatenate two pictures\nSynonyms:\npaint.concat\npaint concat","url":"paint.html#concat"},{"doc":"paint","title":"fill","content":"pub fn fill(picture: Picture, colour: Colour) -&gt; Picture\n Fill a picture with some given colour, see `Colour`.\nSynonyms:\npaint.fill\npaint fill","url":"paint.html#fill"},{"doc":"paint","title":"just","content":"pub fn just(picture: Picture) -&gt; fn(a) -&gt; Picture\n Utility function that is useful for cases where you\n are no interested in the canvas configuration. For example,\n \n canvas.display(just(circle(30.0)), &quot;#my_canvas&quot;)\n // instead of...\n canvas.display(fn(_config) { circle(30.0) }, &quot;#my_canvas&quot;)\n \nSynonyms:\npaint.just\npaint just","url":"paint.html#just"},{"doc":"paint","title":"lines","content":"pub fn lines(points: List(#(Float, Float))) -&gt; Picture\n Lines (same as a polygon but not a closed shape)\nSynonyms:\npaint.lines\npaint lines","url":"paint.html#lines"},{"doc":"paint","title":"polygon","content":"pub fn polygon(points: List(#(Float, Float))) -&gt; Picture\n A polygon consisting of a list of 2d points\nSynonyms:\npaint.polygon\npaint polygon","url":"paint.html#polygon"},{"doc":"paint","title":"rectangle","content":"pub fn rectangle(width: Float, height: Float) -&gt; Picture\n A rectangle with some given width and height\nSynonyms:\npaint.rectangle\npaint rectangle","url":"paint.html#rectangle"},{"doc":"paint","title":"rotate","content":"pub fn rotate(picture: Picture, angle: Angle) -&gt; Picture\n Rotate the picture in a clock-wise direction\nSynonyms:\npaint.rotate\npaint rotate","url":"paint.html#rotate"},{"doc":"paint","title":"scale_uniform","content":"pub fn scale_uniform(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture uniformly in horizontal and vertical direction\nSynonyms:\npaint.scale_uniform\npaint scale_uniform","url":"paint.html#scale_uniform"},{"doc":"paint","title":"scale_x","content":"pub fn scale_x(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture in the horizontal direction\nSynonyms:\npaint.scale_x\npaint scale_x","url":"paint.html#scale_x"},{"doc":"paint","title":"scale_y","content":"pub fn scale_y(picture: Picture, factor: Float) -&gt; Picture\n Scale the picture in the vertical direction\nSynonyms:\npaint.scale_y\npaint scale_y","url":"paint.html#scale_y"},{"doc":"paint","title":"square","content":"pub fn square(length: Float) -&gt; Picture\n A square\nSynonyms:\npaint.square\npaint square","url":"paint.html#square"},{"doc":"paint","title":"stroke","content":"pub fn stroke(\n  picture: Picture,\n  colour: Colour,\n  width width: Float,\n) -&gt; Picture\n Set a solid stroke with some given colour and width\nSynonyms:\npaint.stroke\npaint stroke","url":"paint.html#stroke"},{"doc":"paint","title":"stroke_none","content":"pub fn stroke_none(picture: Picture) -&gt; Picture\n Remove the stroke of the given picture\nSynonyms:\npaint.stroke_none\npaint stroke_none","url":"paint.html#stroke_none"},{"doc":"paint","title":"text","content":"pub fn text(text: String, px font_size: Int) -&gt; Picture\n Text with some given font size\nSynonyms:\npaint.text\npaint text","url":"paint.html#text"},{"doc":"paint","title":"translate_x","content":"pub fn translate_x(picture: Picture, x: Float) -&gt; Picture\n Translate a picture in the horizontal direction\nSynonyms:\npaint.translate_x\npaint translate_x","url":"paint.html#translate_x"},{"doc":"paint","title":"translate_xy","content":"pub fn translate_xy(\n  picture: Picture,\n  x: Float,\n  y: Float,\n) -&gt; Picture\n Translate a picture in horizontal and vertical direction\nSynonyms:\npaint.translate_xy\npaint translate_xy","url":"paint.html#translate_xy"},{"doc":"paint","title":"translate_y","content":"pub fn translate_y(picture: Picture, y: Float) -&gt; Picture\n Translate a picture in the vertical direction\nSynonyms:\npaint.translate_y\npaint translate_y","url":"paint.html#translate_y"},{"doc":"paint","title":"paint","content":" This module contains the main `Picture` type as well as the\n function you can use to construct, modify and combine pictures.","url":"paint.html"},{"doc":"paint/event","title":"Event","content":"pub type Event {\n  Tick(Float)\n  KeyboardPressed(Key)\n  KeyboardRelased(Key)\n  MouseMoved(Float, Float)\n  MousePressed(MouseButton)\n  MouseReleased(MouseButton)\n}\n\nTick(Float)\n Triggered before drawing. Contains the number of milliseconds elapsed.\n\nKeyboardPressed(Key)\n Triggered when a key is pressed\n\nKeyboardRelased(Key)\n Triggered when a key is released\n\nMouseMoved(Float, Float)\n Triggered when the mouse is moved. Contains\n the `x` and `y` value for the mouse position.\n\nMousePressed(MouseButton)\n Triggered when a mouse button is pressed\n\nMouseReleased(MouseButton)\n Triggered when a mouse button is released.\n\n Note, on the web you might encounter issues where the\n release event for the right mouse button is not triggered\n because of the context menu.\n\nSynonyms:\npaint/event.Event\npaint/event Event","url":"paint/event.html#Event"},{"doc":"paint/event","title":"Key","content":"pub type Key {\n  KeyLeftArrow\n  KeyRightArrow\n  KeyUpArrow\n  KeyDownArrow\n  KeySpace\n  KeyW\n  KeyA\n  KeyS\n  KeyD\n  KeyZ\n  KeyX\n  KeyC\n  KeyEnter\n  KeyEscape\n  KeyBackspace\n}\n\nKeyLeftArrow\n\n\nKeyRightArrow\n\n\nKeyUpArrow\n\n\nKeyDownArrow\n\n\nKeySpace\n\n\nKeyW\n\n\nKeyA\n\n\nKeyS\n\n\nKeyD\n\n\nKeyZ\n\n\nKeyX\n\n\nKeyC\n\n\nKeyEnter\n\n\nKeyEscape\n\n\nKeyBackspace\n\n\nSynonyms:\npaint/event.Key\npaint/event Key","url":"paint/event.html#Key"},{"doc":"paint/event","title":"MouseButton","content":"pub type MouseButton {\n  MouseButtonLeft\n  MouseButtonRight\n  MouseButtonMiddle\n}\n\nMouseButtonLeft\n\n\nMouseButtonRight\n\n\nMouseButtonMiddle\n The scroll wheel button\n\nSynonyms:\npaint/event.MouseButton\npaint/event MouseButton","url":"paint/event.html#MouseButton"},{"doc":"paint/event","title":"paint/event","content":" This module contains events that can be triggered when\n building interactive applications.\n\n See `paint/canvas` and the `canvas.interact` function for a\n practical example of how this is used.","url":"paint/event.html"},{"doc":"paint/canvas","title":"Config","content":"pub type Config {\n  Config(width: Float, height: Float)\n}\n The configuration of the &quot;canvas&quot;\nConfig(width: Float, height: Float)\n\n\nSynonyms:\npaint/canvas.Config\npaint/canvas Config","url":"paint/canvas.html#Config"},{"doc":"paint/canvas","title":"center","content":"pub fn center(picture: Picture) -&gt; fn(Config) -&gt; Picture\n Utility to set the origin in the center of the canvas\nSynonyms:\npaint/canvas.center\npaint/canvas center","url":"paint/canvas.html#center"},{"doc":"paint/canvas","title":"define_web_component","content":"pub fn define_web_component() -&gt; Nil\n If you are using [Lustre](https://github.com/lustre-labs/lustre) or some other framework to build\n your web application you may prefer to use the [web components](https://developer.mozilla.org/en-US/docs/Web/API/Web_components) API\n and the `define_web_component` function.\n \n // Call this function to register a custom HTML element &lt;paint-canvas&gt;\n canvas.define_web_component()\n // You can then display your picture by setting the &quot;picture&quot;\n // property on the element.\n\n // In Lustre it would look something like this:\n let my_picture = circle(50.0)\n element(\n   &quot;paint-canvas&quot;,\n    [\n        attribute.property(&quot;picture&quot;, my_picture),\n        attribute.width(500),\n        attribute.height(300),\n        attribute.style([#(&quot;background&quot;, &quot;#eee&quot;)]),\n     ],\n     [],\n )\n \n A more detailed example for using this API together with Lustre can be found in [this GitHub Gist](https://gist.github.com/adelhult/03c5916df891a06bec706e6f0842cd91).\n\nSynonyms:\npaint/canvas.define_web_component\npaint/canvas define_web_component","url":"paint/canvas.html#define_web_component"},{"doc":"paint/canvas","title":"display","content":"pub fn display(\n  init: fn(Config) -&gt; Picture,\n  selector: String,\n) -&gt; Nil\n Display a picture on a HTML canvas element\n (specified by some [CSS Selector](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_selectors)).\n \n canvas.display(fn (_: canvas.Config) { circle(50.0) }, &quot;#mycanvas&quot;)\n \nSynonyms:\npaint/canvas.display\npaint/canvas display","url":"paint/canvas.html#display"},{"doc":"paint/canvas","title":"interact","content":"pub fn interact(\n  init: fn(Config) -&gt; a,\n  update: fn(a, Event) -&gt; a,\n  view: fn(a) -&gt; Picture,\n  selector: String,\n) -&gt; Nil\n Animations, interactive applications and tiny games can be built using the\n `interact` function. It roughly follows the so-called [Elm architecture](https://guide.elm-lang.org/architecture/).\n Here is a short example:\n \n type State =\n   Int\n\n fn init(_: canvas.Config) -&gt; State {\n   0\n }\n\n fn update(state: State, event: event.Event) -&gt; State {\n   case event {\n     event.Tick(_) -&gt; state + 1\n     _ -&gt; state\n   }\n }\n\n fn view(state: State) -&gt; Picture {\n   paint.circle(int.to_float(state))\n }\n\n fn main() {\n   interact(init, update, view, &quot;#mycanvas&quot;)\n }\n \nSynonyms:\npaint/canvas.interact\npaint/canvas interact","url":"paint/canvas.html#interact"},{"doc":"paint/canvas","title":"paint/canvas","content":" A HTML canvas backend that can be used for displaying\n your `Picture`s. There are three different ways of doing so:\n - `display` (provide a picture and a CSS selector to some canvas element)\n - `define_web_component` (an alternative to `display` using custom web components, useful if you are using a web framework like Lustre)\n - `interact` (allows you to make animations and interactive programs)","url":"paint/canvas.html"}]);